using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using Syncfusion.Grouping;
using Syncfusion.Windows.Forms.Grid;

namespace GGC_3_levels_example
{
    public partial class Form1 : Form
    {
        private const string NODE_1 = "1";
        private const string NODE_2 = "2";
        private const string NODE_3 = "3";
        private const string NODE_4 = "4";

        private DataTable parentTable;
        private DataTable table1;
        private DataTable table2;
        private DataTable table2_1;
        private DataTable table3;
        private DataTable table4;

        public Form1()
        {
            InitializeComponent();

            grid.TableModel.Options.ShowCurrentCellBorderBehavior = GridShowCurrentCellBorder.HideAlways;
            grid.TableModel.QueryColWidth += new GridRowColSizeEventHandler( TableModel_QueryColWidth );
        }

        private void TableModel_QueryColWidth(object sender, GridRowColSizeEventArgs e)
        {
            if( e.Index == 3 )
            {
                e.Size = grid.ClientSize.Width - 19;
                e.Handled = true;
            }
        }

        private DataTable GetParentTable()
        {
            DataTable dt = new DataTable("NavParent");


            dt.Columns.Add(new DataColumn("group"));
            dt.Columns.Add(new DataColumn("Description"));

            DataRow dr = dt.NewRow();
            dr[0] = NODE_1;
            dr[1] = "Table1";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr[0] = NODE_2;
            dr[1] = "Table2";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr[0] = NODE_3;
            dr[1] = "Table3";
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr[0] = NODE_4;
            dr[1] = "Table4";
            dt.Rows.Add(dr);

            return dt;
        }

        private DataTable GetTable1()
        {
            DataTable dt = new DataTable("Table1");


            dt.Columns.Add(new DataColumn("Code"));
            dt.Columns.Add(new DataColumn("Name"));
            dt.Columns.Add(new DataColumn("Group"));
            dt.Columns.Add(new DataColumn("ToolTip"));


            DataColumn[] columns = new DataColumn[1];
            columns[0] = dt.Columns["Code"];
            dt.PrimaryKey = columns;

            return dt;
        }

        private DataTable GetTable2()
        {
            DataTable dt = new DataTable("Table2");

            dt.Columns.Add(new DataColumn("Severity"));
            dt.Columns.Add(new DataColumn("Text"));
            dt.Columns.Add(new DataColumn("Group"));
            dt.Columns.Add(new DataColumn("ToolTip"));

            DataColumn[] columns = new DataColumn[1];
            columns[0] = dt.Columns["Text"];
            dt.PrimaryKey = columns;

            DataRow dr = dt.NewRow();
            dr[0] = "1";
            dr[1] = "Test 1";
            dr[2] = NODE_2;
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr[0] = "1";
            dr[1] = "Test 2";
            dr[2] = NODE_2;
            dt.Rows.Add(dr);

            dr = dt.NewRow();
            dr[0] = "1";
            dr[1] = "Test 3";
            dr[2] = NODE_2;
            dt.Rows.Add(dr);
            dr = dt.NewRow();
            dr[0] = "2";
            dr[1] = "Test 4";
            dr[2] = NODE_2;
            dt.Rows.Add(dr);
            dr = dt.NewRow();
            dr[0] = "2";
            dr[1] = "Test 5";
            dr[2] = NODE_2;
            dt.Rows.Add(dr);
            return dt;
        }

        private DataTable GetTable2_1()
        {
            DataTable dt = new DataTable("Table2_1");

            dt.Columns.Add(new DataColumn("Text"));
            dt.Columns.Add(new DataColumn("Code"));
            dt.Columns.Add(new DataColumn("Name"));
            dt.Columns.Add(new DataColumn("ToolTip"));

            DataColumn[] columns = new DataColumn[2];
            columns[0] = dt.Columns["Text"];
            columns[1] = dt.Columns["Code"];
            dt.PrimaryKey = columns;

            DataRow dr;

            for (int j = 1; j < 5; j++)
            {
                for (int i = 0; i < 20; i++)
                {
                    dr = dt.NewRow();
                    dr[0] = "Test " + j;
                    dr[1] = "Code data " + i;
                    dr[2] = "Name data " + i;
                    dt.Rows.Add(dr);
                }
            }

            return dt;
        }

        private DataTable GetTable4()
        {
            DataTable dt = new DataTable("Table4");

            dt.Columns.Add(new DataColumn("Code"));
            dt.Columns.Add(new DataColumn("Name"));
            dt.Columns.Add(new DataColumn("Group"));
            dt.Columns.Add(new DataColumn("ToolTip"));

            DataColumn[] columns = new DataColumn[1];
            columns[0] = dt.Columns["Code"];
            dt.PrimaryKey = columns;
            return dt;
        }

        private DataTable GetTable3()
        {
            DataTable dt = new DataTable("Table3");

            dt.Columns.Add(new DataColumn("Id"));
            dt.Columns.Add(new DataColumn("Name"));
            dt.Columns.Add(new DataColumn("Group"));
            dt.Columns.Add(new DataColumn("ToolTip"));

            DataColumn[] columns = new DataColumn[1];
            columns[0] = dt.Columns["Id"];
            dt.PrimaryKey = columns;

            return dt;
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            parentTable = GetParentTable();
            table1 = GetTable1();
            table2 = GetTable2();
            table2_1 = GetTable2_1();
            table3 = GetTable3();
            table4 = GetTable4();

            DataSet ds = new DataSet("MyDataSet");
            ds.Tables.AddRange(
                new DataTable[]
                    {parentTable, table1, table2, table2_1, table4, table3});

            DataRelation parentToChild1 =
                new DataRelation("ParentToChild1", parentTable.Columns["group"], table1.Columns["Group"]);
            ds.Relations.AddRange(new DataRelation[] {parentToChild1});

            DataRelation parentToChild2 =
                new DataRelation("ParentToChild2", parentTable.Columns["group"], table2.Columns["Group"]);
            ds.Relations.AddRange(new DataRelation[] {parentToChild2});

            DataRelation parentToChild3 =
                new DataRelation("ParentToChild3", parentTable.Columns["group"], table4.Columns["Group"]);
            ds.Relations.AddRange(new DataRelation[] {parentToChild3});

            DataRelation parentToChild4 =
                new DataRelation("ParentToChild4", parentTable.Columns["group"], table3.Columns["Group"]);
            ds.Relations.AddRange(new DataRelation[] {parentToChild4});

            DataRelation Table2ToTable2_1 =
                new DataRelation("Table2ToTable2_1", table2.Columns["Text"],
                                 table2_1.Columns["Text"]);
            ds.Relations.AddRange(new DataRelation[] {Table2ToTable2_1});

            grid.DataSource = parentTable;
        }

        private void grid_TableControlDrawCell(object sender,
                                               Syncfusion.Windows.Forms.Grid.Grouping.GridTableControlDrawCellEventArgs
                                                   e)
        {
            if (e.TableControl.TableDescriptor.Name.Equals("NavParent") && e.Inner.ColIndex == 3)
            {
                e.Inner.Renderer.Draw(e.Inner.Graphics, e.Inner.Bounds, e.Inner.RowIndex, e.Inner.ColIndex,
                                      e.Inner.Style);

                Icon failureIcon = SystemIcons.Error;
                Icon warningIcon = SystemIcons.Warning;

                Rectangle iRect =
                    new Rectangle(e.Inner.Bounds.Width - 100, e.Inner.Bounds.Y, e.Inner.Bounds.Width,
                                  e.Inner.Bounds.Height);
                e.Inner.Graphics.DrawIconUnstretched(warningIcon, iRect);

                iRect.X = iRect.X + 80;
                e.Inner.Graphics.DrawIconUnstretched(failureIcon, iRect);

                e.Inner.Cancel = true;
            }
            else if( e.TableControl.TableDescriptor.Name.Equals( "Table2" ) && e.Inner.ColIndex == 2 )
            {
                Icon icon = null;

                Record r = grid.Table.RelatedTables["Table2"].DisplayElements[e.Inner.RowIndex].ParentRecord;

                if( r.GetValue( "Severity" ).ToString( ).Equals( "1" ) )
                    icon = SystemIcons.Warning;

                else if( r.GetValue( "Severity" ).ToString( ).Equals( "2" ) )
                    icon = SystemIcons.Error;

                if( icon != null )
                {
                    e.Inner.Renderer.Draw( e.Inner.Graphics, e.Inner.Bounds, e.Inner.RowIndex, e.Inner.ColIndex,
                                          e.Inner.Style );

                    Rectangle iRect =
                        new Rectangle( e.Inner.Bounds.X, e.Inner.Bounds.Y, e.Inner.Bounds.Width,
                                      e.Inner.Bounds.Height );

                    e.Inner.Graphics.DrawIconUnstretched( icon, iRect );

                    e.Inner.Cancel = true;
                }
            }
        }
    }
}